; Death Trap v1.1 for TSE
; By Robin Kay
;

; You may not distribute a modified version of this program. However, you
; may take code from or base your program on this code providing that the
; resulting program is at least 50% of your own work.

.nolist
#include "tse.inc"
.list
.org    usermem-3
.db     $bb,$6d
 ret

.db "TSE",1              ; header
prgtitle:
.db " Death Trap", 0     ; title
.dw 67                   ; stack required

start:

 ; Dispaly main menu
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl 
 ld hl, prgtitle + 1
 call putStr
 ld hl, author
 call putStr
 bcall(_newline)
 ld hl, txt1
 call putStr
 ld hl, txt2
 call putStr
 bcall(_newline)
 bcall(_newline)
 ld hl, hstxt
 bcall(_puts)
 ld hl, (highscore)
 bcall(_disphl)

 ; Poll for keypad on menu
loop:
 ei
 halt ; Save power
 bcall(_getcsc)

 cp sk1
 jr nz, cont0
 ld h, 15
 jr maingame
cont0:

 cp sk2
 jr nz, cont1
 ld h, 5
 jr maingame
cont1:

 cp sk3
 jr nz, cont2
 ld h, 2
 jr maingame
cont2:

 ; Process other keys
 cp skclear
 jr z, clrhs

 ; Yield
 cp skStat
 jr nz, dontyield
 call _tseforceyield
 jr start
dontyield:
 jr loop

; Clear highscore
clrhs:
 ld hl, 0
 ld (highscore), hl
 jr start 

; Main game routine
maingame:

 ; Save delay
 ld l, 0
 ld (delay), hl

 ; Setup spike locations
 ld hl, spikes
 ld b, 12
setuploop:
 push bc
 ld b, 128
 call ionrandom
 add a, 64
 ld (hl), a
 inc hl
 pop bc
 djnz setuploop

 ; Init car position and score
 ld a, 48-4
 ld (car_pos), a
 ld hl, 0
 ld (score), hl

redrawgame:

 ; Clear the graph buffer
 bcall(_GrBufClr)

 ; Redraw missing trails
 ld hl, spikes + 12
 ld b, 12
redrawloop:
 dec hl
 push hl
 push bc

 ld a, (hl)
 inc a
 cp 60
 jr nc, skipReDraw
 dec b
 ld l, b
 ld h, 0
 ld de, plotsscreen - 12
 add hl, de
 ld de, 12
 ld b, a
redrawloop2:
 add hl, de
 ld (hl), 0ffh
 djnz redrawloop2
skipReDraw:

 pop bc
 pop hl
 djnz redrawloop

loopgame:

 ; Delay
 ld bc, (delay)
 ld hl, 8000h
 ld d, h
 ld e, l
 ldir ; Nop!

 ; Increase score
 ld hl, (score)
 inc hl
 ld (score), hl

 ; Clear spikes
 ld b, 12
 ld hl, spikes + 12
clearLoop:
 dec hl
 push bc
 push hl

 ; Check if spike is at bottom of display
 dec b
 ld a, (hl)
 cp 59
 jr nz, skipClr

 ; Clear spike from display
 push hl
 ld l, b
 ld h, 0
 ld de, plotsscreen - 12
 add hl, de
 ld b, 64
 ld de, 12
 xor a
loop2:
 add hl, de
 ld (hl), a
 djnz loop2
 pop hl

 ; Re-Randomize spike y
 ld b, (hl)
 ld a, r
 and %01111111
 add a, b
 ld (hl), a

 : Loop
skipClr:
 pop hl
 pop bc
 djnz clearLoop 

 ; Draw spikes
 ld b, 12
 ld hl, spikes + 12
drawloop:
 dec hl
 push bc
 push hl
 dec b
 ld a, (hl)
 inc a
 cp 192
 jr nz, notAbove
 xor a
notAbove:
 ld (hl), a
 ld l, b
 ld b, a
 cp 60
 jr nc, dontDraw

 ; Draw spike onto display
 ld h, 0
 ld de, plotsscreen - 12
 add hl, de
 ld de, 12
 inc b
loop1:
 add hl, de
 djnz loop1
 ld (hl), %11111111
 add hl, de
 ld (hl), %01111110
 add hl, de
 ld (hl), %00111100
 add hl, de
 ld (hl), %00011000

 ; Loop
dontDraw:
 pop hl
 pop bc
 djnz drawloop

 ; Process keypad
 ld a,kreset
 out (1),a
 ld a,group1
 out (1),a
 in a,(1)
 cp kright
 jr z, right
 cp kleft
 jr nz, nothing
left:
 ld a, (car_pos)
 dec a
 cp -1
 jr z, nothing
 ld (car_pos), a
 jr nothing
right:
 ld a, (car_pos)
 inc a
 cp 96-8
 jr z, nothing
 ld (car_pos), a
nothing:

 ; Check for collision
 ld a, (car_pos)
 add a, 4
 sra a
 sra a
 sra a
 ld c, a
 ld b, 0
 ld hl, spikes
 add hl, bc
 ld a, (hl)
 cp 56
 jr c, belowCar
 cp 60
 jr c, done
belowCar:

 ; Draw car
 call xorCar

 ; Redraw display
 call ionfastcopy

 ; Clear car
 call xorCar

 ; Yield
 ld a,kreset
 out (1),a
 ld a,group4
 out (1),a
 in a,(1)
 cp KStat
 jp nz, loopgame
 call _tseforceyield
 jp redrawgame

; Hiscore screen
done:
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl
 ld hl, scrtxt
 bcall(_puts)
 ld hl, (score)
 bcall(_disphl)
 bcall(_newline)
 ld hl, (score)
 ld de, (highscore)
 inc de
 bcall(_cphlde)
 jr c, nohighscore

 ld (highscore), hl
 ld hl, newhs
 bcall(_puts)

nohighscore:
 ei
 halt ; Save power
 bcall(_getcsc)
 cp skMode
 jp z, start
 cp skStat
 jr nz, nohighscore
 call z, _tseforceyield
 jr done

; Display string and then advance a line onwards
putStr:
 bcall(_puts)
 bcall(_newline)
 ret

; XOR the car onto the screen
xorCar:
 ld a, (car_pos)
 ld l, 64-4
 ld b, 5
 ld ix, car
 call ionputsprite
 ret

highscore:
.dw 0

car:
.db %00011000
.db %00111100
.db %01111110
.db %11000011
.db %10000001

author:
.db "By Robin Kay", 0
txt1:
.db "1-3: Start Game",0
txt2:
.db "Clr: Reset HS",0
hstxt:
.db "Hi-"
scrtxt
.db "Score: ",0
newhs:
.db "New HighScore!!",0

dta:
spikes  .equ dta
delay   .equ spikes + 12
car_pos .equ delay + 2
score   .equ car_pos + 1

.end
